import { initWs } from "@/config/holo/websocket";
import { convertToLineWithWkt } from "@/utils/mapboxTools";
import * as mapTools from "@/utils/mapboxTools";
import { getDetetors } from "@/dao/situation";
export default {
    data() {
        return {
            detetorsStatus: false,
            detetors: {},
        }
    },
    mounted() {

    },
    methods: {
        initDetetorData() {
            // 初始化所有检测器位置信息
            getDetetors().then((res) => {
                // console.log("检测器位置信息。。。", res);
                this.detetorsArray = res.content;
                this.detetors = {};
                for (let item of res.content) {
                    if (!this.detetors[item.detectorType]) {
                        this.detetors[item.detectorType] = [];
                    }
                    this.detetors[item.detectorType].push({ ...item, state: "0" });
                }
                for (let key in this.detetors) {
                    let geo = convertToLineWithWkt(this.detetors[key]);
                    let type = key.slice(key.length - 1, key.length);
                    mapTools[`addOrUpdateDetetor${type}`](map, geo);
                }
            });
        },
        unRegistDetetors() {
            Promise.all([
                this.closeWs("callDetetorA"),
                this.closeWs("callDetetorB"),
                this.closeWs("callDetetorC"),
            ]).then(() => {
                this.removeLayers("detetorsA,detetorsB,detetorsC");
                this.detetorsStatus = false;
            });
        },
        registDetetors() {
            if (!this.detetorsStatus) {
                let wsUrl= ws_config.BASE_URL
                // let wsUrl = `ws://${window.wsHost}/holows/subscribe`
                let ws1 = initWs({
                    name: "callDetetorA",
                    url: wsUrl,
                    callback: (e) => this.callDetetor(e, "A"),
                });
                let ws2 = initWs({
                    name: "callDetetorB",
                    url: wsUrl,
                    callback: (e) => this.callDetetor(e, "B"),
                });
                let ws3 = initWs({
                    name: "callDetetorC",
                    url: wsUrl,
                    callback: (e) => this.callDetetor(e, "C"),
                });
                this.$nextTick(() => {
                    if (!Object.keys(this.detetors).length) {
                        this.initDetetorData()
                    }
                });
                ws1.ws.onopen = () => {
                    if (ws1.ws.readyState === 1) {
                        ws1.ws.send('{ dataType: "DETECTOR_A"}');
                        this.sockets.push(ws1);
                    }
                };
                ws2.ws.onopen = () => {
                    if (ws2.ws.readyState === 1) {
                        ws2.ws.send('{ dataType: "DETECTOR_B"}');
                        this.sockets.push(ws2);
                    }
                };
                ws3.ws.onopen = () => {
                    if (ws3.ws.readyState === 1) {
                        ws3.ws.send('{ dataType: "DETECTOR_C"}');
                        this.sockets.push(ws3);
                    }
                };
                this.detetorsStatus = true;
            }
        },
        callDetetor(msg, type) {
            // 因为接收到检测器个数不完整
            // 需要与所有检测器信息聚合 后 再更新图层 避免缺失
            // id 只在 全量数据 中存在且不在 接收数据 中存在的即为 需要合并的数据
            if (this.detetors[`DETECTOR_${type}`]) {
                let del = this.detetors[`DETECTOR_${type}`].filter(
                    (e) => !msg.find((c) => c.laneId === e.laneId)
                );
                for (let item of msg) {
                    let match = this.detetors[`DETECTOR_${type}`].find((de) => {
                        return de.laneId === item.laneId;
                    });
                    item.wkt = match?.wkt;
                }
                let currentAll = [...msg, ...del];
                let geo = convertToLineWithWkt(currentAll);
                mapTools[`addOrUpdateDetetor${type}`](map, geo);
            }
        },
    },
    beforeDestroy() {
        this.detetors = {}
        this.detetorsStatus = false
    }
}