import { equip_radar } from "@/dao/situation";
import * as mapTools from "@/utils/mapboxTools";
let radarModelTypes = {}
let radarTimers = []
let radarModels = {}
export default {
    data() {
        return {
            radarShow: false,
            radarDatas: [],
            radarLayers: [],
            radarSources: [],
        }
    },
    mounted() {
    },
    methods: {
        initRadarModels() {
            for (let item of [1, 2]) {
                window.tb.loadObj(
                    {
                        obj: `gltf/radar${item}.glb`,
                        type: "gltf",
                        units: "meters",
                        scale: 10,
                        adjustment: { x: 0.5, y: 1, z: -0.5 },
                        bbox: false,
                    },
                    (model) => {
                        radarModelTypes[item] = model;
                    }
                );
            }
        },
        loadRadars() {
            if (this.radarDatas.length) {
                this.solveRadarDatas();
            } else {
                Promise.all([equip_radar()]).then(([radar]) => {
                    this.radarDatas = radar.content;
                    this.solveRadarDatas();
                });
            }
        },
        solveRadarDatas() {
            let milliFeatures = [];
            console.log("radars...", this.radarDatas);
            let timer = setInterval(() => {
                if (radarModelTypes[1] && radarModelTypes[2]) {
                    clearInterval(timer)
                    for (let i = 0; i < this.radarDatas.length; i++) {
                        //2 毫米波  other 激光雷达
                        let model =
                            radarModelTypes[this.radarDatas[i].sourceEquipType].duplicate();
                        model.setCoords(this.radarDatas[i].wkt.split(","));
                        model.userData.data = this.radarDatas[i];
                        model.setRotation({ x: 90, y: 360 - 90, z: 0 });
                        window.tb.add(
                            model,
                            `radar${i}${this.radarDatas[i].sourceEquipType}`
                        );
                        radarModels[`radar${i}${this.radarDatas[i].sourceEquipType}`] = model;
                        if (this.radarDatas[i].sourceEquipType == 1) {
                            this.addRadarTimers(this.radarDatas[i], i);
                        } else {
                            let lnglat = this.radarDatas[i].wkt.split(",");
                            let center = turf.point(lnglat);
                            let radius = 0.17; // 100米半径
                            let direct = Number(this.radarDatas[i].equipmentDirect);
                            let arc = turf.lineArc(center, radius, direct - 10, direct + 10, {
                                unit: "kilometers",
                            });
                            let arcScope = [[lnglat, ...arc.geometry.coordinates, lnglat]];
                            milliFeatures.push(turf.polygon(arcScope, this.radarDatas[i]));
                        }
                    }
                    let milliGeo = turf.featureCollection(milliFeatures);
                    mapTools.addOrUpdateMilliScope(window.map, milliGeo);
                }
            }, 1000)
        },
        addRadarTimers(radarData, index) {
            this.radarShow = true;
            const numRings = 4; // 水波的层数
            const maxMeters = 200; // 最大半径，例如100米
            let lnglat = radarData.wkt.split(",");
            const maxPx = this.convertPxToMeter(maxMeters, window.map.getZoom(), lnglat[1]);
            for (let ii = 0; ii <= numRings; ii++) {
                const radius = (ii / numRings) * maxPx;
                const opacity = 1 - ii / numRings;
                if (!window.map.getSource(`circle-source-${ii}-${index}`)) {
                    window.map.addSource(`circle-source-${ii}-${index}`, {
                        type: "geojson",
                        data: {
                            type: "Feature",
                            geometry: {
                                type: "Point",
                                coordinates: lnglat,
                            },
                            properties: {
                                size: radius,
                            },
                        },
                    });
                    this.radarSources.push(`circle-source-${ii}-${index}`);
                }
                if (!window.map.getLayer(`circle-layer-${ii}-${index}`)) {
                    window.map.addLayer(
                        {
                            id: `circle-layer-${ii}-${index}`,
                            source: `circle-source-${ii}-${index}`,
                            type: "circle",
                            paint: {
                                "circle-radius": radius,
                                "circle-opacity": 0,
                                "circle-stroke-color": "#f6eee0",
                                "circle-stroke-opacity": opacity,
                                "circle-stroke-width": 2,
                                "circle-pitch-alignment": "map",
                            },
                        },
                        "vehicle3D"
                    );
                    this.radarLayers.push(`circle-layer-${ii}-${index}`);
                }
            }
            let timer = setInterval(() => {
                if (window.map && this.radarLayers.length) {
                    let maxPxInterval = this.convertPxToMeter(
                        maxMeters,
                        window.map.getZoom(),
                        lnglat[1]
                    );
                    for (let ii = 0; ii < numRings; ii++) {
                        let radius = window.map.getPaintProperty(
                            `circle-layer-${ii}-${index}`,
                            "circle-radius"
                        );
                        if (radius < maxPxInterval - 1) {
                            window.map.setPaintProperty(
                                `circle-layer-${ii}-${index}`,
                                "circle-radius",
                                radius + 3
                            );
                            window.map.setPaintProperty(
                                `circle-layer-${ii}-${index}`,
                                "circle-stroke-opacity",
                                1 - radius / maxPxInterval < 0 ? 0 : 1 - radius / maxPxInterval
                            );
                        } else {
                            window.map.removeLayer(`circle-layer-${ii}-${index}`);
                            window.map.addLayer(
                                {
                                    id: `circle-layer-${ii}-${index}`,
                                    source: `circle-source-${ii}-${index}`,
                                    type: "circle",
                                    paint: {
                                        "circle-radius": 0,
                                        "circle-opacity": 0,
                                        "circle-stroke-color": "#f6eee0",
                                        "circle-stroke-opacity": 1,
                                        "circle-stroke-width": 2,
                                        "circle-pitch-alignment": "map",
                                    },
                                },
                                "vehicle3D"
                            );
                        }
                    }
                } else {
                    clearInterval(timer);
                }
            }, 45 - window.map.getZoom() * 2);
            radarTimers.push(timer);
        },
        loadUpdateRadars() {
            if (this.shouldShowRadar) {
                if (!this.radarShow) {
                    this.radarShow = true;
                    this.loadRadars();
                } else {
                    this.updateRadars()
                }
            }
        },
        updateRadars() {
            this.clearRadarWave();
            setTimeout(() => {
                for (let i = 0; i < this.radarDatas.length; i++) {
                    if (this.radarDatas[i].sourceEquipType == 1) {
                        this.addRadarTimers(this.radarDatas[i], i);
                    }
                }
            });
        },
        removeAllRadars() {
            this.clearRadarWave()
            this.removeLayers("milliScope");
            for (let key in radarModels) {
                window.tb.clear(key, null);
            }
        },
        clearRadarWave() {
            for (let tim of radarTimers) {
                clearInterval(tim);
            }
            for (let layer of this.radarLayers) {
                this.removeLayers(layer);
            }
            for (let source of this.radarSources) {
                if (window.map.getSource(source)) window.map.removeSource(source);
            }
            this.radarShow = false;

            this.radarLayers = [];
            this.radarSources = [];

        },
    }
}