let map = null, draw = null
export default {
	name: "drawTestUtil",
	components: {},
	data() {
		return {
			centers: [],
		};
	},
	props: [],
	watch: {},
	mounted() { },
	beforeDestroy() {
		if (map) {
			map.off('mousemove', this.mousemove)
		}
	},
	methods: {
		updateDistanceLabel(features, allFeatures) {
			if (features && features.length && allFeatures && allFeatures.length) {
				this.centers = []
				let geojson = allFeatures;
				for (let item of geojson) {
					let coords = item.geometry.type === 'LineString' ? item.geometry.coordinates : item.geometry.coordinates[0]
					for (let i = 0; i < coords.length - 1; i++) {
						this.centers.push(
							Object.assign(
								{},
								turf.midpoint(turf.point(coords[i]), turf.point(coords[i + 1])),
								{
									properties: {
										distance: ((turf.distance(turf.point(coords[i]), turf.point(coords[i + 1]), { units: 'kilometers' })) * 1000).toFixed(2) + 'm'
									}
								}
							)
						)
					}
					// area
					if (item.geometry.type !== "LineString") {
						this.centers.push(
							Object.assign(
								{},
								turf.center(turf.featureCollection(features)),
								{
									properties: {
										distance: (turf.area(item)).toFixed(2) + '㎡'
									}
								}
							)
						)
					}
				}
				let centersGeo = turf.featureCollection(this.centers)
				if (!map.getSource("distanceLabel")) {
					map.addSource("distanceLabel", {
						type: "geojson",
						data: centersGeo,
					});
				} else {
					map.getSource("distanceLabel").setData(centersGeo);
				}
				if (!map.getLayer("distanceLabelText")) {
					map.addLayer(
						{
							id: "distanceLabelText",
							type: "symbol",
							source: "distanceLabel",
							layout: {
								"text-field": ["get", "distance"],
								"text-size": 10,
								'text-allow-overlap': true,
								'text-ignore-placement': true
							},
							paint: {
								"text-color": "white",
								"text-halo-blur": 1,
								"text-halo-color": 'black',
								"text-halo-width": 2,
							},
						},
					);
					map.moveLayer('distanceLabelText')
				}
			}
		},
		mixDraw(receiveMap) {
			map = receiveMap
			draw = new MapboxDraw({
				displayControlsDefault: false,
				controls: {
					polygon: true,
					line_string: true,
					point: false,
					trash: true,
				},
			});
			map.addControl(draw);
			map.on('mousemove', this.mousemove)
			map.on('draw.delete', () => {
				if (map && map.getLayer('distanceLabelText')) {
					map.removeLayer('distanceLabelText')
				}
			})
		},
		mousemove() {
			if (draw) {
				// console.log('move', draw.getAll());
				let geo = draw.getAll()
				if (geo.features?.length) {
					let newGeo = JSON.parse(JSON.stringify({ type: 'FeatureCollection', features: [geo.features[geo.features.length - 1]] }))
					this.updateDistanceLabel(newGeo.features, geo.features)
				}
			}
		}
	},
};
